==> token-client-kotlin-demo-4.1.4-sources.jar: no/nav/security/token/support/ktor/oauth/OAuth2Cache.kt <==
package no.nav.security.token.support.ktor.oauth

import com.github.benmanes.caffeine.cache.AsyncLoadingCache
import com.github.benmanes.caffeine.cache.Caffeine
import com.github.benmanes.caffeine.cache.Expiry
import java.util.concurrent.TimeUnit
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.future.future
import no.nav.security.token.support.client.core.oauth2.OAuth2AccessTokenResponse

data class OAuth2CacheConfig(val enabled: Boolean, val maximumSize: Long = 1000, val evictSkew: Long = 5) {
    fun cache(cacheContext: CoroutineScope, loader: suspend (GrantRequest) -> OAuth2AccessTokenResponse): AsyncLoadingCache<GrantRequest, OAuth2AccessTokenResponse> =
        Caffeine.newBuilder()
            .expireAfter(evictOnResponseExpiresIn(evictSkew))
            .maximumSize(maximumSize)
            .buildAsync { key, _ ->
                cacheContext.future {
                    loader(key)
                }
            }

    private fun evictOnResponseExpiresIn(skewInSeconds: Long): Expiry<GrantRequest, OAuth2AccessTokenResponse> {
        return object : Expiry<GrantRequest, OAuth2AccessTokenResponse> {

            override fun expireAfterCreate(key: GrantRequest, response: OAuth2AccessTokenResponse, currentTime: Long): Long {
                val seconds =
                    if (response.expiresIn!! > skewInSeconds) response.expiresIn!! - skewInSeconds else response.expiresIn!!
                        .toLong()
                return TimeUnit.SECONDS.toNanos(seconds)
            }

            override fun expireAfterUpdate(key: GrantRequest, response: OAuth2AccessTokenResponse, currentTime: Long, currentDuration: Long): Long = currentDuration


            override fun expireAfterRead(key: GrantRequest, response: OAuth2AccessTokenResponse, currentTime: Long, currentDuration: Long): Long = currentDuration
        }
    }
}

==> token-client-kotlin-demo-5.0.10-sources.jar: no/nav/security/token/support/ktor/oauth/OAuth2Cache.kt <==
package no.nav.security.token.support.ktor.oauth

import com.github.benmanes.caffeine.cache.AsyncLoadingCache
import com.github.benmanes.caffeine.cache.Caffeine
import com.github.benmanes.caffeine.cache.Expiry
import java.util.concurrent.TimeUnit
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.future.future
import no.nav.security.token.support.client.core.oauth2.OAuth2AccessTokenResponse

data class OAuth2CacheConfig(val enabled: Boolean, val maximumSize: Long = 1000, val evictSkew: Long = 5) {
    fun cache(cacheContext: CoroutineScope, loader: suspend (GrantRequest) -> OAuth2AccessTokenResponse): AsyncLoadingCache<GrantRequest, OAuth2AccessTokenResponse> =
        Caffeine.newBuilder()
            .expireAfter(evictOnResponseExpiresIn(evictSkew))
            .maximumSize(maximumSize)
            .buildAsync { key, _ ->
                cacheContext.future {
                    loader(key)
                }
            }

    private fun evictOnResponseExpiresIn(skewInSeconds: Long): Expiry<GrantRequest, OAuth2AccessTokenResponse> {
        return object : Expiry<GrantRequest, OAuth2AccessTokenResponse> {

            override fun expireAfterCreate(key: GrantRequest, response: OAuth2AccessTokenResponse, currentTime: Long): Long {
                val seconds =
                    if (response.expires_in!! > skewInSeconds) response.expires_in!! - skewInSeconds else response.expires_in!!
                        .toLong()
                return TimeUnit.SECONDS.toNanos(seconds)
            }

            override fun expireAfterUpdate(key: GrantRequest, response: OAuth2AccessTokenResponse, currentTime: Long, currentDuration: Long): Long = currentDuration


            override fun expireAfterRead(key: GrantRequest, response: OAuth2AccessTokenResponse, currentTime: Long, currentDuration: Long): Long = currentDuration
        }
    }
}
